@@ -19,14 +19,17 @@ AccessTypeDict = {
             "rdfs:comment": "The dandiset is embargoed",
             "rdfs:label": "Embargoed",
         },
-        {
-            "@id": "dandi:RestrictedAccess",
-            "@type": "dandi:AccessType",
-            "rdfs:comment": "The dandiset is restricted",
-            "rdfs:label": "Restricted",
-        },
     ]
 }
+"""
+Uncomment and add to accesstype dict when restricted access is implemented
+{
+    "@id": "dandi:RestrictedAccess",
+    "@type": "dandi:AccessType",
+    "rdfs:comment": "The dandiset is restricted",
+    "rdfs:label": "Restricted",
+},
+"""
 
 DigestTypeDict = {
     "@graph": [
@@ -79,6 +82,12 @@ DigestTypeDict = {
             "rdfs:comment": "S3-style ETag",
             "rdfs:label": "DANDI ETag",
         },
+        {
+            "@id": "dandi:dandi-zarr-checksum",
+            "@type": "dandi:DigestType",
+            "rdfs:comment": "DANDI Zarr checksum",
+            "rdfs:label": "DANDI Zarr",
+        },
     ]
 }
 
@@ -144,12 +153,6 @@ LicenseTypeDict = {
             "@type": "dandi:LicenseType",
             "rdfs:label": "Attribution 4.0 International (CC BY 4.0)",
         },
-        {
-            "@id": "spdx:CC-BY-NC-4.0",
-            "rdfs:seeAlso": "https://creativecommons.org/licenses/by-nc/4.0/legalcode",
-            "@type": "dandi:LicenseType",
-            "rdfs:label": "Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)",
-        },
     ]
 }
 
